/******************************************************************************
 *                                                                            *
 * File:   pcstat32.c         Version  1.01            Date: 2000-01-31       *
 *                                                                            *
 * Copyright (C) 1999-2000 by Kostis Netzwerkberatung                         *
 * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056, FAX 881058            *
 * kosta@kostis.net (Kosta Kostis), http://www.kostis.net/                    *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *        - display data about *this* machine in XML format                   *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     2000-01-31: KK V1.01                                                   *
 *        - added patch for Win95(a)                                          *
 *          -DWIN95A to compile Win95(a) version                              *
 *     1999-12-19: KK V1.00                                                   *
 *        - initial encoding                                                  *
 *                                                                            *
 *****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <nb30.h>

#include "pcstat32.h"

/******************************************************************************

	Function:
		display copyright, author and program information

	Parameters:
		none

	Returns:
		nothing

 *****************************************************************************/

static  void    Hello (void)
{
	fprintf (stderr, MSG_COPYRIGHT) ;
	fprintf (stderr, MSG_VERSION, PRGNAM, VERSION) ;
}

/******************************************************************************

	Function:
		return CPUid for Win95 since it does not supply this info

	Parameters:
		none

	Returns:
		3,4,5,6 (-86)

 *****************************************************************************/

int	w95cpuid (void)
{
	DWORD	cpu ;

__asm
	{
		pushf			; save flag register content
		push	eax		;

		mov	cpu,3		; we got i386 or above

		pushfd			; check out EFLAG register
		pop	eax		;

		mov	ecx,eax		; make a safe copy in ecx
		xor	eax,40000h	; try to toggle bit #18 (AC)

		push	eax		; from eax to EFLAGS and back
		popfd			;
		pushfd			;
		pop	eax		;

		xor	eax,ecx		; check AC bit
		jz	done_386	; if it can't be changed, it's i386

; -- let's check for CPUID instruction ------------------------

		mov	cpu,4		; it's i486 or better

		mov	eax,ecx		; get original EFLAGS
		xor	eax,200000h	; try to toggle bit #21

		push	eax		; from eax to EFLAGS and back
		popfd			;
		pushfd			;
		pop	eax		;

		xor	eax,ecx		; check ID bit
		jz	done_386	; if it can't be changed, no CPUID

		push	ecx		; save ecx for now
		push	ebx		; and ebx also

		xor	eax,eax		; now retrieve information
		_emit	0x0f		; hardcoded opcode for CPUID instruction
		_emit	0xa2		;
		cmp	eax,1		; is 1 a valid input?
		jl	done_486	; no, let's leave it alone

		xor	eax,eax		;
		inc	eax		; eax=1, get family, model, stepping
		_emit	0x0f		; hardcoded opcode for CPUID instruction
		_emit	0xa2		;
		and	eax,0F00h	; get CPU family (4-6)
		shr	eax,8		; shift it to the correct position
		mov	cpu,eax		; copy value to cpu

done_486:	pop	ebx		; restore ecx and ebx
		pop	ecx		;

done_386:	push	ecx		; restore flags to original state
		popfd			;

pend:		pop	eax		;
		popf			; restore inital flag register
	}

	return (cpu) ;
}

/******************************************************************************

	Function:
		convert binary MAC address into human readable form (netdb)

	Parameters:
		pointer to MAC address

	Returns:
		MAC address in netdb format

 *****************************************************************************/

static	char	*MAC2char
(
	UCHAR	*a
)
{
	static	char	MAC	[MAC_MAXLEN + 1] ;

	sprintf (MAC, "%02X-%02X-%02X-%02X-%02X-%02X",
		a [0], a [1], a [2], a [3], a [4], a [5]) ;

	return (MAC) ;
}

/******************************************************************************

	Function:
		retrieve MAC address of LANA0

	Parameters:
		none

	Returns:
		char	*		MAC address in netdb format

 *****************************************************************************/

char	*GetMAC (void)
{
	char		*help = (char *) NULL ;
	NCB		ncb ;
	LANA_ENUM	lenum ;
	PADAPTER_STATUS	pas ;
	int		i ;
	int		rc ;
	int		cas ;

	/**********************************************************************

		fetch memory for data structures

	 *********************************************************************/

	cas = sizeof (ADAPTER_STATUS) + (sizeof (NAME_BUFFER) * MAX_LANA) ;
	pas = (PADAPTER_STATUS) malloc (cas) ;
	if (pas == (PADAPTER_STATUS) NULL)
		return (help) ;
	ZeroMemory (pas, cas) ;

	/**********************************************************************

		reset adapter

	 *********************************************************************/

	ZeroMemory (&ncb, sizeof (NCB)) ;
	ncb.ncb_command  = NCBRESET ;
	ncb.ncb_lana_num = 0 ;
	Netbios (&ncb) ;

	/**********************************************************************

		retrieve adapter status

	 *********************************************************************/

	ZeroMemory (&ncb, sizeof (NCB)) ;
	ncb.ncb_command  = NCBASTAT ;
	memset (ncb.ncb_name, ' ', NCBNAMSZ - 1) ;
	memset (ncb.ncb_callname, ' ', NCBNAMSZ - 1) ;
	ncb.ncb_callname [0] = '*' ;
	ncb.ncb_lana_num = 0 ;
	ncb.ncb_buffer   = (PUCHAR) pas ;
	ncb.ncb_length   = cas ;
	Netbios (&ncb) ;

	/**********************************************************************

		retrieve MAC address and convert into netdb format

	 *********************************************************************/

	help = MAC2char (pas -> adapter_address) ;

	/**********************************************************************

		free memory for data structures

	 *********************************************************************/

	free ((void *) pas) ;

	return (help) ;
}

/******************************************************************************

	main program retrieving data about the PC it runs on

 *****************************************************************************/

int     main
(
	int     	argc,
	char		*argv	[]
)
{
	// variables for GetComputerName ()
	char		cn		[COMPUTERNAME_MAXLEN + 1] ;
	DWORD		cnlen 		= COMPUTERNAME_MAXLEN ;

	// variable for WSAStartup ()
	WSADATA		wd ;

	// variable for gethostname ()
	char		hn		[HOSTNAME_MAXLEN + 1] ;

	// variable for gethostbyname ()
	struct	hostent	*he ;

	// variables for GetUserName ()
	char		un		[USERNAME_MAXLEN + 1] ;
	DWORD		unlen		= USERNAME_MAXLEN ;

	// variable for GetVersionEx ()
	OSVERSIONINFO	osvi ;

	// variable for GetLocalTime () ;
	SYSTEMTIME	st ;

	// variable for GetSysInfo ()
	SYSTEM_INFO	sinfo ;

	// variable for GlobalMemoryStatus ()
	MEMORYSTATUS	ms ;

	// variable for GetDriveType ()
	UINT		dt ;

	// variables for GetDiskFreeSpace () - Win95(a)
	DWORD		SectorsPerCluster ;
	DWORD		BytesPerSector ;
	DWORD		FreeClusters ;
	DWORD		Clusters ;

	// variables for GetDiskFreeSpaceEx ()
	char		RootPathName	[DRIVENAME_MAXLEN + 1] ;
	ULARGE_INTEGER	FreeBytesAvailableToCaller ;
	ULARGE_INTEGER	TotalNumberOfBytes ;
	ULARGE_INTEGER	TotalNumberOfFreeBytes ;

	// variables for GetVolumeInformation
	char		szVolumeName	[LABEL_MAXLEN + 1] ;
	DWORD		dwMaxCompLen ;
	DWORD		dwFileSysFlags ;
	DWORD		dwVolSerialNum ;

	// generic variables
	BOOL		rc		= TRUE ;	// return code
	char		drive ;				// loop through drives
	int		i ;				// loop through ipaddr
	unsigned char	*ipaddr ;
	char		lastdrive ;

	/**********************************************************************

		parse parameters/arguments

	 *********************************************************************/

	switch (argc)
	{
		case	1:

			break ;


		case	2:

			if (strcmp (argv [1], "-csv") == 0)
				break ;

		default:

			/******************************************************

				display syntax error message and quit

			 *****************************************************/

			Hello () ;
			fprintf (stderr, ERR_MSG_SYNTAX, PRGNAM) ;

			return (ERR_SYNTAX) ;
	}

	// retrieve computername (convert to lower case)
	GetComputerName (cn, &cnlen) ;
	CharLower (cn) ;

	// retrieve and display OS version
	osvi.dwOSVersionInfoSize = sizeof (osvi) ;
	GetVersionEx (&osvi) ;

	// display XML 1.0 header
	if (argc == 1)
		printf ("<?xml version=\"1.0\" standalone=\"yes\"?>\n") ;

	// display computername (lower case)
	if (argc == 1)
		printf ("<%s>\n", cn) ;
	else
		printf ("\"%s\"", cn) ;

	// display os information
	if (argc == 1)
	{
		printf ("\t<os>\n") ;
		printf ("\t\t<osplatform>") ;
	}
	else
	{
		printf (",\"") ;
	}

	switch (osvi.dwPlatformId)
	{
		case	VER_PLATFORM_WIN32_WINDOWS:

			if (osvi.dwMinorVersion < 10)
				printf ("95") ;
			else
				printf ("98") ;	// >= 4.10
			break ;

		case	VER_PLATFORM_WIN32_NT:

			printf ("NT") ;
			break ;
	}
	if (argc == 1)
		printf ("</osplatform>\n") ;
	else
		printf ("\"") ;

	if (argc == 1)
		printf ("\t\t<osversion>%u.%u</osversion>\n",
			osvi.dwMajorVersion, osvi.dwMinorVersion) ;
	else
		printf (",\"%u.%u\"",
			osvi.dwMajorVersion, osvi.dwMinorVersion) ;

	if ((osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) && (argc == 1))
	{
		printf ("\t\t<osbuild>%u</osbuild>\n", osvi.dwBuildNumber) ;
		printf ("\t\t<oscsdversion>%s</oscsdversion>\n", osvi.szCSDVersion) ;
	}

	// end of os information
	if (argc == 1)
	{
		printf ("\t</os>\n") ;
		printf ("\t<net>\n") ;
	}

	// retrieve and display username
	if (GetUserName (un, &unlen) == 0)
		strcpy (un, "") ;

	if (argc == 1)
		printf ("\t\t<user>%s</user>\n", un) ;
	else
		printf (",\"%s\"", un) ;

	// retrieve and display hostname and IP address
	WSAStartup (0x0101, &wd) ;		// init winsock
	gethostname (hn, HOSTNAME_MAXLEN) ;	// whoami
	if (argc == 1)
		printf ("\t\t<hostname>%s</hostname>\n", hn) ;
	he = gethostbyname (hn) ;		// ip addr lookup
	if (he != (struct hostent *) NULL)
	{
		if (argc == 1)
		{
			printf ("\t\t<fqdn>%s</fqdn>\n", he -> h_name) ;
			printf ("\t\t<ipaddr>") ;
		}
		else
		{
			printf (",\"") ;
		}

		ipaddr = he -> h_addr ;
		for (i = 0 ; i < 4 ; ++i)
		{
			if (i != 0)
				printf (".") ;
			printf ("%u", ipaddr [i]) ;
		}

		if (argc == 1)
			printf ("</ipaddr>\n") ;
		else
			printf ("\"") ;
	}

	// retrieve and display MAC address
	if (argc == 1)
		printf ("\t\t<mac>%s</mac>\n", GetMAC ()) ;
	else
		printf (",\"%s\"", GetMAC ()) ;

	// end of network information
	if (argc == 1)
		printf ("\t</net>\n") ;

	// retrieve and display date and time
	GetLocalTime (&st) ;
	if (argc == 1)
	{
		printf ("\t<date>\n") ;
		printf ("\t\t<localdate>%04u-%02u-%02u</localdate>\n",
			st.wYear, st.wMonth, st.wDay) ;
		printf ("\t\t<localtime>%02u:%02u:%02u</localtime>\n",
			st.wHour, st.wMinute, st.wSecond) ;
		printf ("\t</date>\n") ;
	}
	else
	{
		printf (",\"%04u-%02u-%02u\"",
			st.wYear, st.wMonth, st.wDay) ;
		printf (",\"%02u:%02u:%02u\"",
			st.wHour, st.wMinute, st.wSecond) ;
	}

	// retrieve system details (cpu)
	GetSystemInfo (&sinfo) ;
	if (sinfo.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL)
	{
		if (argc == 1)
			printf ("\t<cpu>\n") ;

		// display number of CPUs
		if (argc == 1)
			printf ("\t\t<cpunum>%u</cpunum>\n", sinfo.dwNumberOfProcessors) ;

		// fix bug in Microsoft Windows 95 implementation of WIN32
		if (osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
			if (osvi.dwMinorVersion < 10)
				if (sinfo.wProcessorLevel > 9)
					sinfo.wProcessorLevel = w95cpuid () ;

		// display processor type
		if (argc == 1)
			printf ("\t\t<cpuid>%u86</cpuid>\n", sinfo.wProcessorLevel) ;
		else
			printf (",\"%u86\"",  sinfo.wProcessorLevel) ;

		// depending on ProcessorLevel give more details
		switch (sinfo.wProcessorLevel)
		{
			case	3:	// i386
			case	4:	// i486

				break ;

			default:

				if (argc == 1)
				{
					printf ("\t\t<cpumodel>%u</cpumodel>\n",
						sinfo.wProcessorRevision >> 8) ;
					printf ("\t\t<cpustepping>%u</cpustepping>\n",
						sinfo.wProcessorRevision & 0xFF) ;
				}
		}
	}
	if (argc == 1)
		printf ("\t</cpu>\n") ;

	// retrieve and display amount of physical memory
	GlobalMemoryStatus (&ms) ;

	// correct dwTotalPhys since it seems to be 1 MB short
	if (argc == 1)
		printf ("\t<ram>%u</ram>\n", (ms.dwTotalPhys / MB) + 1) ;
	else
		printf (",\"%u\"",  (ms.dwTotalPhys / MB) + 1) ;

	// display disk drive space details
	// start with drive c: and ignore all "errors"
	if (argc == 1)
		printf ("\t<drives>\n") ;
	strcpy (RootPathName, DRV_C) ;			/* init path */
	lastdrive = 'z' ;

	// in CSV mode check only drive C:
	if (argc != 1)
		lastdrive = 'c' ;

	for (drive = 'c' ; drive <= lastdrive ; ++drive)
	{
		RootPathName [0] = drive ;		/* driver letter */

		dt = GetDriveType (RootPathName) ;	/* check drive */
		if (dt != DRIVE_FIXED)			/* skip non-fixed */
			continue ;

		if (argc == 1)
		{
			printf ("\t\t<drive>\n") ;
			printf ("\t\t\t<driveletter>%c</driveletter>\n", drive) ;
		}

		// label information
		GetVolumeInformation (RootPathName,
			szVolumeName, (DWORD) LABEL_MAXLEN,
			&dwVolSerialNum, &dwMaxCompLen,
			&dwFileSysFlags, (LPTSTR) NULL, (DWORD) 0) ;
		if (argc == 1)
			printf ("\t\t\t<drivelabel>%s</drivelabel>\n", szVolumeName) ;
		else
			printf (",\"%s\"", szVolumeName) ;

		// disk space (total and free) information
#ifdef WIN95A
		GetDiskFreeSpace (RootPathName,
			&SectorsPerCluster,
			&BytesPerSector,
			&FreeClusters,
			&Clusters) ;
		if (argc == 1)
		{
			printf ("\t\t\t<drivesize>%lu</drivesize>\n",
				(Clusters * SectorsPerCluster * BytesPerSector) / MB) ;
			printf ("\t\t\t<drivefree>%lu</drivefree>\n",
				(FreeClusters * SectorsPerCluster * BytesPerSector) / MB) ;
		}
		else
		{
			printf (",\"%lu\"\n",
				(Clusters * SectorsPerCluster * BytesPerSector) / MB) ;
			printf (",\"%lu\"\n",
				(FreeClusters * SectorsPerCluster * BytesPerSector) / MB) ;
		}
#else
		GetDiskFreeSpaceEx (RootPathName,
			&FreeBytesAvailableToCaller,
			&TotalNumberOfBytes,
			&TotalNumberOfFreeBytes) ;
		if (argc == 1)
		{
			printf ("\t\t\t<drivesize>%lu</drivesize>\n",
				TotalNumberOfBytes.QuadPart / MB) ;
			printf ("\t\t\t<drivefree>%lu</drivefree>\n",
				TotalNumberOfFreeBytes.QuadPart / MB) ;
			printf ("\t\t</drive>\n") ;
		}
		else
		{
			printf (",\"%lu\"", TotalNumberOfBytes.QuadPart / MB) ;
			printf (",\"%lu\"\n", TotalNumberOfFreeBytes.QuadPart / MB) ;
		}
#endif
	}

	if (argc == 1)
	{
		printf ("\t</drives>\n") ;
		printf ("</%s>\n", cn) ;
	}

	return (ERR_NONE) ;
}
